@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2016, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -80,6 +80,10 @@ import stem.util.str_tools
 
 from stem.util import log
 
+# lines to limit our trace logging to, you can disable this by setting it to None
+
+TRUNCATE_LOGS = 10
+
 
 class ControlSocket(object):
   """
@@ -122,14 +126,14 @@ class ControlSocket(object):
           raise stem.SocketClosed()
 
         send_message(self._socket_file, message, raw)
-      except stem.SocketClosed as exc:
+      except stem.SocketClosed:
         # if send_message raises a SocketClosed then we should properly shut
         # everything down
 
         if self.is_alive():
           self.close()
 
-        raise exc
+        raise
 
   def recv(self):
     """
@@ -154,7 +158,7 @@ class ControlSocket(object):
           raise stem.SocketClosed()
 
         return recv_message(socket_file)
-      except stem.SocketClosed as exc:
+      except stem.SocketClosed:
         # If recv_message raises a SocketClosed then we should properly shut
         # everything down. However, there's a couple cases where this will
         # cause deadlock...
@@ -174,7 +178,7 @@ class ControlSocket(object):
             self.close()
             self._send_lock.release()
 
-        raise exc
+        raise
 
   def is_alive(self):
     """
@@ -208,7 +212,7 @@ class ControlSocket(object):
   def connection_time(self):
     """
     Provides the unix timestamp for when our socket was either connected or
-    disconnected. That is to say, the time we connected if we're presently
+    disconnected. That is to say, the time we connected if we're currently
     connected and the time we disconnected if we're not connected.
 
     .. versionadded:: 1.3.0
@@ -517,7 +521,7 @@ def recv_message(control_file):
       a complete message
   """
 
-  parsed_content, raw_content = [], b''
+  parsed_content, raw_content = [], []
   logging_prefix = 'Error while receiving a control message (%s): '
 
   while True:
@@ -547,7 +551,7 @@ def recv_message(control_file):
       log.info(prefix + 'received exception "%s"' % exc)
       raise stem.SocketClosed(exc)
 
-    raw_content += line
+    raw_content.append(line)
 
     # Parses the tor control lines. These are of the form...
     # <status code><divider><content>\r\n
@@ -574,6 +578,7 @@ def recv_message(control_file):
 
     line = line[:-2]  # strips off the CRLF
     status_code, divider, content = line[:3], line[3:4], line[4:]
+    content_lines = [content]
 
     if stem.prereq.is_python_3():
       status_code = stem.util.str_tools._to_unicode(status_code)
@@ -586,10 +591,18 @@ def recv_message(control_file):
       # end of the message, return the message
       parsed_content.append((status_code, divider, content))
 
-      log_message = raw_content.replace(b'\r\n', b'\n').rstrip()
-      log.trace('Received from tor:\n' + stem.util.str_tools._to_unicode(log_message))
+      raw_content_str = b''.join(raw_content)
+      log_message = stem.util.str_tools._to_unicode(raw_content_str.replace(b'\r\n', b'\n').rstrip())
 
-      return stem.response.ControlMessage(parsed_content, raw_content)
+      if TRUNCATE_LOGS:
+        log_message_lines = log_message.split('\n')
+
+        if len(log_message_lines) > TRUNCATE_LOGS:
+          log_message = '\n'.join(log_message_lines[:TRUNCATE_LOGS] + ['... %i more lines...' % (len(log_message_lines) - TRUNCATE_LOGS)])
+
+      log.trace('Received from tor:\n' + log_message)
+
+      return stem.response.ControlMessage(parsed_content, raw_content_str)
     elif divider == '+':
       # data entry, all of the following lines belong to the content until we
       # get a line with just a period
@@ -599,14 +612,14 @@ def recv_message(control_file):
           line = stem.util.str_tools._to_bytes(control_file.readline())
         except socket.error as exc:
           prefix = logging_prefix % 'SocketClosed'
-          log.info(prefix + 'received an exception while mid-way through a data reply (exception: "%s", read content: "%s")' % (exc, log.escape(raw_content)))
+          log.info(prefix + 'received an exception while mid-way through a data reply (exception: "%s", read content: "%s")' % (exc, log.escape(b''.join(raw_content))))
           raise stem.SocketClosed(exc)
 
-        raw_content += line
+        raw_content.append(line)
 
         if not line.endswith(b'\r\n'):
           prefix = logging_prefix % 'ProtocolError'
-          log.info(prefix + 'CRLF linebreaks missing from a data reply, "%s"' % log.escape(raw_content))
+          log.info(prefix + 'CRLF linebreaks missing from a data reply, "%s"' % log.escape(b''.join(raw_content)))
           raise stem.ProtocolError('All lines should end with CRLF')
         elif line == b'.\r\n':
           break  # data block termination
@@ -619,13 +632,12 @@ def recv_message(control_file):
         if line.startswith(b'..'):
           line = line[1:]
 
-        # appends to previous content, using a newline rather than CRLF
-        # separator (more conventional for multi-line string content outside
-        # the windows world)
+        content_lines.append(line)
 
-        content += b'\n' + line
+      # joins the content using a newline rather than CRLF separator (more
+      # conventional for multi-line string content outside the windows world)
 
-      parsed_content.append((status_code, divider, content))
+      parsed_content.append((status_code, divider, b'\n'.join(content_lines)))
     else:
       # this should never be reached due to the prefix regex, but might as well
       # be safe...
